/* 
   copied from http://www.captain.at/xenomai-timer-task.php

   then modified: combined 2 progs, to eliminate common code, hilite
   sleeper and waiter tasks
*/

#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <sys/mman.h>	// mlockall 
#include <sys/time.h>	// gettimeofday

#define STACK_SIZE 8192
#define STD_PRIO 1
#define MILLION		1000000

int int_count = 0;
int end = 0;

/* options */
int do_mode = 0;  // 0waiter-task
int do_set_mode = 0;  // waiter-task mod

struct timeval tp;
int quit_now = 0;

#include <native/task.h>
RT_TASK test_task_ptr;
RTIME delay_ns = 10 * MILLION;

void sleeper_task(void *cookie) {
	int count = 0;

	// gettimeofday(&tp, 0);

	printf("starting sleeper_task ct=%d \n", count);
	while (!end) {
 		rt_task_sleep(delay_ns);
		count++;
		printf("print by sleeper_task ct=%d \n", count);
		fflush(NULL);
	}
}

#include <native/alarm.h>
RT_ALARM dl_alarm;

#define ALARM_TIME 500000
#define ALARM_INTERVAL 250000

static void alarm_task(void *cookie)
{
    int i = 0;
    int ret = rt_alarm_create(&dl_alarm, "alarm0") ;
    if (ret) {
        printf("\ncannot create alarm %d\n", ret);
	return;
    }
    rt_alarm_start(&dl_alarm, ALARM_TIME, ALARM_INTERVAL);

    for (i = 0; i<100; i++) {
        rt_alarm_wait(&dl_alarm);
        printf("i = %d\n",i);
    }

    quit_now = 1;
    printf("quit_now is set to %d\n",quit_now);
    return;
}

void waiter_task(void *cookie) {
	int count = 0;
	int ret;
	unsigned long overruns;

	ret = rt_task_set_periodic(NULL, TM_NOW, rt_timer_ns2ticks(delay_ns));

	if (ret) {
		printf("error while set periodic, code %d\n",ret);
		return;
	}

	while (!end) {
		if (do_set_mode) {
			ret = rt_task_set_mode(0, T_PRIMARY, NULL);
			if (ret) {
				printf("error while rt_task_set_mode, code %d\n",ret);
				return;
			}
		}
		ret = rt_task_wait_period(&overruns);
		if (ret) {
			printf("error while rt_task_wait_period, code %d\n",ret);
			return;
		}
		count++;
		printf("print by waiter_task: count=%d\n", count);
		fflush(NULL);
	}
}

// signal-handler, to ensure clean exit on Ctrl-C

void clean_exit(int sig)
{
	printf("cleanup %d\n", sig);
	// rt_alarm_stop(&dl_alarm); recent add
	end = 1;
	rt_task_delete(&test_task_ptr);
	printf("end\n");
}

int main(int argc, char *argv[]) {
	int err;
	char c;
	void *taskp = &sleeper_task;

	printf("main\n");

	while ((c = getopt(argc,argv,"T:swa")) != EOF) {

		switch (c) {
		case 'T':
			delay_ns = MILLION * atoi(optarg);
			break;
		case 's':
			do_set_mode = 1;
			break;
		case 'w':
			taskp = &waiter_task;
			break;
		case 'a':
			taskp = &alarm_task;
			break;
		default:
			printf("err: unknown option %c\n",c);
		}
	}

	mlockall(MCL_CURRENT|MCL_FUTURE);

	// install signal handlers
	signal(SIGTERM, clean_exit);	
	signal(SIGINT, clean_exit);	

	err = rt_task_spawn(&test_task_ptr, "task-timer", STACK_SIZE,
			    STD_PRIO, 0, taskp, NULL);
	if (err) {
		printf("error rt_task_spawn\n");
		return 0;
	}
	// wait for signal & return of signal handler
	pause();
	fflush(NULL);
	return 0;
}
