/*
  http://www.captain.at/xenomai-timer-task.php
  Example #1 - timer.c
*/

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <sys/time.h>
#include <sys/io.h>
#include <sys/mman.h>
#include <native/task.h>
#include <native/queue.h>
#include <native/intr.h>

#define STACK_SIZE 8192
#define STD_PRIO 1
#define MILLION		1000000

RT_TASK test_task_ptr;
RTIME sleep_for_ns = 1000 * MILLION;
int end = 0;

/* options */
int do_waiter = 0;  // waiter-task
int do_set_mode = 0;  // waiter-task mod

void waiter_task(void *cookie) {
	int count = 0;
	int ret;
	unsigned long overruns;

	ret = rt_task_set_periodic(NULL, TM_NOW, rt_timer_ns2ticks(sleep_for_ns));

	if (ret) {
		printf("error while set periodic, code %d\n",ret);
		return;
	}

	while (!end) {
		if (do_set_mode) {
			ret = rt_task_set_mode(0, T_PRIMARY, NULL);
			if (ret) {
				printf("error while rt_task_set_mode, code %d\n",ret);
				return;
			}
		}
		ret = rt_task_wait_period(&overruns);
		if (ret) {
			printf("error while rt_task_wait_period, code %d\n",ret);
			return;
		}
		count++;
		printf("message from waiter_task: count=%d\n", count);
		fflush(NULL);
	}
}

void clean_exit(int dummy) {
	printf("cleanup %d\n", dummy);
	end = 1;
	rt_task_delete(&test_task_ptr);
	printf("end\n");
}

int main(int argc, char *argv[]) {
	int err;
	char c;
	printf("main\n");

	while ((c = getopt(argc,argv,"S:s")) != EOF) {

		switch (c) {
		case 'S':
			sleep_for_ns = MILLION * atoi(optarg);
			break;
		case 's':
			do_set_mode = 1;
			break;
		}
	}
	mlockall(MCL_CURRENT|MCL_FUTURE);

	// install signal handler
	signal(SIGTERM, clean_exit);	
	signal(SIGINT, clean_exit);	

	err = rt_task_spawn(&test_task_ptr, "Timer", STACK_SIZE, 
			    STD_PRIO, 0, &waiter_task, NULL);
	if (err) {
		printf("error rt_task_spawn\n");
		return 0;
	}
	// wait for signal & return of signal handler
	pause();
	fflush(NULL);
	return 0;
}

