/* 
   http://www.captain.at/xenomai-timer-task.php
   timer2.c
*/

#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <sys/mman.h>	// mlockall 
#include <sys/time.h>	// gettimeofday
#include <native/task.h>

#define STACK_SIZE 8192
#define STD_PRIO 1
#define MILLION		1000000

RT_TASK test_task_ptr;
RTIME delay_ns = 10 * MILLION;
int int_count = 0;
int end = 0;

/* options */
int do_waiter = 0;  // waiter-task
int do_set_mode = 0;  // waiter-task mod

struct timeval tp;

void sleeper_task(void *cookie) {
	int count = 0;

	// gettimeofday(&tp, 0);

	printf("starting sleeper_task ct=%d \n", count);
	while (!end) {
 		rt_task_sleep(delay_ns);
		count++;
		printf("print by sleeper_task ct=%d \n", count);
		fflush(NULL);
	}
}

void waiter_task(void *cookie) {
	int count = 0;
	int ret;
	unsigned long overruns;

	ret = rt_task_set_periodic(NULL, TM_NOW, rt_timer_ns2ticks(delay_ns));

	if (ret) {
		printf("error while set periodic, code %d\n",ret);
		return;
	}

	while (!end) {
		if (do_set_mode) {
			ret = rt_task_set_mode(0, T_PRIMARY, NULL);
			if (ret) {
				printf("error while rt_task_set_mode, code %d\n",ret);
				return;
			}
		}
		ret = rt_task_wait_period(&overruns);
		if (ret) {
			printf("error while rt_task_wait_period, code %d\n",ret);
			return;
		}
		count++;
		printf("message from waiter_task: count=%d\n", count);
		fflush(NULL);
	}
}

// signal-handler, to ensure clean exit on Ctrl-C
void clean_exit(int dummy) {
	printf("cleanup %d\n",dummy);
	end = 1;
	rt_task_delete(&test_task_ptr);
	printf("end\n");
}

int main(int argc, char *argv[]) {
	int err;
	char c;
	void *taskp;

	printf("main\n");

	while ((c = getopt(argc,argv,"T:st:")) != EOF) {

		switch (c) {
		case 'T':
			delay_ns = MILLION * atoi(optarg);
			break;
		case 's':
			do_set_mode = 1;
			break;
		case 't':
			do_waiter = atoi(optarg);
			break;
		}
	}

	if (do_waiter == 1)
		taskp = &waiter_task;
	else 
		taskp = &sleeper_task;



	mlockall(MCL_CURRENT|MCL_FUTURE);

	// install signal handlers
	signal(SIGTERM, clean_exit);	
	signal(SIGINT, clean_exit);	

	err = rt_task_spawn(&test_task_ptr, "Timer", STACK_SIZE,
			    STD_PRIO, 0, taskp, NULL);
	if (err) {
		printf("error rt_task_spawn\n");
		return 0;
	}
	// wait for signal & return of signal handler
	pause();
	fflush(NULL);
	return 0;
}
