#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <getopt.h>

#include <sys/mman.h>
#include <rtai/task.h>
#include <rtai/alarm.h>

#define TASK_PRIO  99 /* Highest RT priority */
#define TASK_MODE  0  /* No flags */
#define TASK_STKSZ 0  /* Stack size (use default one) */

#define ALARM_INITDELAY	500000	/* First shot at now + 500 us */
#define ALARM_INTERVAL	250000	/* Period is 250 us */

RT_ALARM alarm_desc;
RT_TASK server_desc;
RTIME	sleeptime = 100000000;
int	mainsleep = 5;

void cleanup_upon_sig (int sig)
{
  int err;
  err = rt_alarm_stop(&alarm_desc);
  if (err) printf("err %d: alarm_stop\n", err);

  err = rt_alarm_delete(&alarm_desc);
  if (err) printf("err %d: alarm_delete\n", err);

  err = rt_task_delete(&server_desc);
  if (err) printf("err %d: task_delete\n", err);

  printf("alarm stopped, deleted, task deleted\n");
}

void alarm_server (void *cookie)
{
  int i=0;
    for (;;) {
       /* Wait for the next alarm to trigger. */
      int err = rt_alarm_wait(&alarm_desc);
      switch (err) {
      case 0: 
	break;
      case -EINVAL:
	printf("%d: alarm is not an alarm descriptor\n", err);
	break;
      case -EPERM:
	printf("%d: called from a context which cannot sleep\n", err);
	break;
      case -EIDRM:
	printf("%d: alarm is a deleted alarm descriptor\n", err);
	break;
      case -EINTR:
	printf("%d: task interrupted before the next alarm shot\n", err);
	break;
      default:
	printf("%d: timer start undoc'd err\n", err);
      }
      printf("%d ", i++);
    }
}

int main (int argc, char *argv[])
{
    int err, c;
    RTIME period;

    signal(SIGINT, cleanup_upon_sig);
    signal(SIGTERM, cleanup_upon_sig);
    signal(SIGHUP, cleanup_upon_sig);
    signal(SIGALRM, cleanup_upon_sig);

    while ((c = getopt(argc,argv,"op:s:")) != EOF)
        switch (c) {
	case 'o':
	  err = rt_timer_start(TM_ONESHOT);
	  if (err)
	    printf("error setting oneshot\n");
	  else 
	    printf("timer started oneshot\n");
	  break;
	case 'p':
	  period = atoi(optarg);
	  err = rt_timer_start(period);
	  switch (err) {
	  case -EINVAL:
	    printf("%d: timer start, einval\n", err);
	    break;
	  default:
	    printf("%d: timer started periodic: %Ld \n", err, period);
	  }
	  break;
	case 's':
	  mainsleep = atoi(optarg);
	  break;
	default:
	  printf("dunno option\n");
	}


    mlockall(MCL_CURRENT|MCL_FUTURE);

    // err = rt_timer_start(TM_ONESHOT);

    // printf("timer started oneshot\n");


    err = rt_alarm_create(&alarm_desc,"MyAlarm");
    switch (err) {
    case 0:
      printf("alarm created\n");
      break;
    case -ENOMEM:
      printf("%d: alarm register failed, enomem\n", err);
      break;
    case -EEXIST: 
      printf("%d: alarm register failed, eexist\n", err);
      break;
    case -EPERM:
      printf("%d: alarm register failed, eperm\n", err);
      break;
    default:
      printf("%d: alarm_create undoc'd err\n", err);
    }


    err = rt_task_create(&server_desc,
			 "MyAlarmServer",
			 TASK_STKSZ,
			 TASK_PRIO,
			 TASK_MODE);
    if (err)
      printf("%d: rt_task-create\n", err);
    else
      printf("rt_task created\n");

    err = rt_task_start(&server_desc,&alarm_server,NULL);
    if (err)
      printf("%d: rt_task-create\n", err);
    else
      printf("rt_task created\n");


    err = rt_alarm_start(&alarm_desc,
			 ALARM_INITDELAY,
			 ALARM_INTERVAL);
    if (err)
      printf("%d: rt_alarm_start\n", err);
    else
      printf("rt_alarm started\n");

    sleep(mainsleep);
    cleanup_upon_sig(err);
    return err;

}

