#include <stdio.h>	// printf
#include <stdlib.h>	// atoi
#include <getopt.h>
#include <signal.h>
#include <sys/mman.h>
#include <rtai/task.h>


#define TASK_PRIO  99 /* Highest RT priority */
#define TASK_MODE  0  /* No flags */
#define TASK_STKSZ 0  /* Stack size (use default one) */

RT_TASK task_desc;
RTIME	sleeptime = 100000000;
int	finished = 0;

void cleanup_upon_sig (int sig)
{
  int err = 0;

  /* without setting finished somewhere, this handler wont kill the
     program, apparently cuz the task interrupts the system call, and
     hence the handler.  Once flag is set, task stops looping, and the
     interrupted system call is properly restarted.
  */
  finished = 1;
  /* Unfortunately, finishing here means the 'cat' doesnt show the
     task in the /proc/rtai files
  */
  printf("finishing task, heres /proc/rtai/{sched,stats}:\n");
  system("cat /proc/rtai/{sched,stat}");
  if (err)
    perror("system call failed\n");

  finished = 1;	// to late - 

  //err = rt_task_stop(&server_desc);
  //if (err) printf("err %d: task_stop\n", err);

  err = rt_task_delete(&task_desc);
  if (err) printf("err %d: task_delete\n", err);

  printf("task stopped, deleted\n");
  exit(0);
}

void task_body (void *cookie)
{
  int err, i = 0;
  /* ... "cookie" should be NULL  (is... */

    for (;;) {
      if (finished) break;
      printf("in task body: %d\n", i++);
      err = rt_task_sleep(sleeptime);
      switch (-err) {
      case 0:
	break;
      case EINTR:
	printf("rt_task_sleep interrupted\n");
	break;
      case EWOULDBLOCK:
	printf("rt_task_sleep would block, system timer inactive: %d\n", err);
	break;
      case EPERM:
	printf("rt_task_sleep not allowed\n");
	break;
      default:
	printf("rt_task_sleep undoc'd err: %d\n", err);
      }
    }
}

int main (int argc, char *argv[])
{
    int err, c;
    RTIME period;

    signal(SIGINT, cleanup_upon_sig);
    signal(SIGTERM, cleanup_upon_sig);
    signal(SIGHUP, cleanup_upon_sig);
    signal(SIGALRM, cleanup_upon_sig);

    while ((c = getopt(argc,argv,"op:s:")) != EOF)
        switch (c) {
	case 'o':
	  err = rt_timer_start(TM_ONESHOT);
	  if (err)
	    printf("error setting oneshot\n");
	  break;
	case 'p':
	  period = atoi(optarg);
	  err = rt_timer_start(period);
	  if (err)
	    printf("error setting periodic %Ld: %d\n", period, err);
	  break;
	case 's':
	  sleeptime = atoi(optarg);
	  break;
	default:
	  printf("dunno option\n");
	}


    mlockall(MCL_CURRENT|MCL_FUTURE);

    /* ... */

    err = rt_task_create(&task_desc,
			 "MyTaskName",
			 TASK_STKSZ,
			 TASK_PRIO,
			 TASK_MODE);
    if (!err)
      err = rt_task_start(&task_desc,&task_body,NULL);

    if (err)
      printf("error starting task\n");

    /* ... */
    
    return err;
}

